@@ -15,7 +15,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""Sugar content bundles"""
+"""Sugar content bundles
+
+UNSTABLE.
+"""
 
 from ConfigParser import ConfigParser
 import os
@@ -49,6 +52,7 @@ class ContentBundle(Bundle):
         self._category_icon = None
         self._library_version = None
         self._bundle_class = None
+        self._activity_start = None
 
         info_file = self.get_file('library/library.info')
         if info_file is None:
