@@ -17,32 +17,27 @@
 
 """
 All the constants are expressed in pixels. They are defined for the XO screen
-and are usually adapted to different resolution by applying a zoom factor. The
-factor for traditional 96 dpi screen is currently 0.72 which is the inverse
-of the one we are using to adapt web pages to the XO screen. It should be
-considered a reference value rather then a scale constant which has to be
-automatically applied and always respected.
+and are usually adapted to different resolution by applying a zoom factor.
+
+STABLE.
 """
 
 import os
+import logging
 
 import gtk
 import pango
 
-_XO_DPI = 200.0
-
 _FOCUS_LINE_WIDTH = 2
 _TAB_CURVATURE = 1
 
-def _get_screen_dpi():
-    xft_dpi = gtk.settings_get_default().get_property('gtk-xft-dpi')
-    return float(xft_dpi / 1024)
-
 def _compute_zoom_factor():
-    if _get_screen_dpi() == 96.0:
-        if not os.environ.has_key('SUGAR_XO_STYLE') or \
-           not os.environ['SUGAR_XO_STYLE'] == 'yes':
-            return 0.72
+    if os.environ.has_key('SUGAR_SCALING'):
+        try:
+            scaling = int(os.environ['SUGAR_SCALING'])
+            return scaling / 100.0
+        except ValueError:
+            logging.error('Invalid SUGAR_SCALING.')
 
     return 1.0
 
@@ -112,7 +107,7 @@ MEDIUM_ICON_SIZE = zoom(55 * 1.5)
 LARGE_ICON_SIZE = zoom(55 * 2.0)
 XLARGE_ICON_SIZE = zoom(55 * 2.75)
 
-FONT_SIZE = zoom(7 * _XO_DPI / _get_screen_dpi())
+FONT_SIZE = zoom(7)
 FONT_NORMAL = Font('Bitstream Vera Sans %d' % FONT_SIZE)
 FONT_BOLD = Font('Bitstream Vera Sans bold %d' % FONT_SIZE)
 FONT_NORMAL_H = zoom(24)
